"use strict";

function CalcStateMach() {
	var disp = "0",
		acc = "0",
		currOp = "+",
		resetDisp = false,
		temp = "";
	this.screen = disp;
	this.state = [disp, acc, currOp];
	this.push = function(inp) {
		switch (inp) {
			case "C":
				disp = "0";
				acc = "0";
				currOp = "+";
				resetDisp = false;
				break;
			case "E":
				disp = "0";
				acc = acc;
				currOp = currOp;
				resetDisp = true;
				break;
			case "=":
				temp = disp;
				disp = solve(acc, disp, currOp);
				if (resetDisp) {
					acc = acc;
				} else {
					acc = temp;
				}
				currOp = currOp;
				resetDisp = true;
				break;
			case "0":
			case "1":
			case "2":
			case "3":
			case "4":
			case "5":
			case "6":
			case "7":
			case "8":
			case "9":
				if (resetDisp) { disp = "0"; }
				disp = (disp + inp).replace(/^0(?=\d)/, "");
				if (disp.indexOf(".") === -1) {
					disp = disp.slice(0,8);
				} else {
					disp = disp.slice(0,9);
				}				
				acc = acc;
				currOp = currOp;
				resetDisp = false;
				break;
			case ".":
				if (resetDisp) { disp = "0"; }
				if (disp.indexOf(".") === -1) {
					disp = disp + ".";
				}
				acc = acc;
				currOp = currOp;
				resetDisp = false;
				break;
			case "+":
			case "-":
			case "*":
			case "/":
				if (resetDisp) {
					disp = disp;
					acc = disp;
				} else {
					temp = solve(acc, disp, currOp);
					disp = temp;
					acc = temp;
				}
				currOp = inp;
				resetDisp = true;
				break;
			default:
				break;
		}
		
		// if input results in error, reset
		if (disp === "E") {
			acc = "0", currOp = "+";
		}
		
		this.state = [disp, acc, currOp];
		this.screen = disp;
	};
	
	var solve = function(n, m, op) {
		n = parseFloat(n);
		m = parseFloat(m);
		if (op === "+") { return formatDisplay(n + m); }
		if (op === "-") { return formatDisplay(n - m); }
		if (op === "/") { return formatDisplay(n / m); }
		if (op === "*") { return formatDisplay(n * m); }
	};
}

// formatDisplay: returns display-styled
// string from number.
var formatDisplay = function (n) {
	// Error messages
	var DIVIDE_BY_ZERO = "E";
	var NAN_ERROR = "E";
	var TOO_SMALL = "E";
	var TOO_SMALL_NEG = "E";
	var OVERFLOW = "E";
	var UNDERFLOW = "E";

	var DISPLAY_MAX = 99999999.5;
	var DISPLAY_MIN = -99999999.5;
	var DISPLAY_SMALLEST = 0.00000005;
	var DISPLAY_SMALLEST_NEG = -0.00000005;

	// formatting errors
	if (isNaN(n)) { return NAN_ERROR; }
	if (n === Number.POSITIVE_INFINITY) { return DIVIDE_BY_ZERO; }
	if (n === Number.NEGATIVE_INFINITY) { return DIVIDE_BY_ZERO; }
	if (n >= DISPLAY_MAX) { return OVERFLOW; }
	if (n < DISPLAY_MIN) { return UNDERFLOW; }
	if (n > 0 && n < DISPLAY_SMALLEST) { return TOO_SMALL; }
	if (n < 0 && n > DISPLAY_SMALLEST_NEG) { return TOO_SMALL_NEG; }

	var decimalPlace = parseInt(n.toExponential().split("e")[1], 10) + 1,
	signif = n.toExponential()
					.split("e")[0]
					.replace(/[\.\-]/, "")
					.length;
	if (signif <= 8 && decimalPlace > -6 && decimalPlace <= 8) {
		// no formatting necessary
		return n.toString();
	} else {
		n = n.toFixed(8 - decimalPlace);
		var extra = 0;
		if (n.indexOf(".") !== -1) { extra++; }
		if (n.indexOf("-") !== -1) { extra++; }
		return n.slice(0, 8 + extra);
	}
};
